@@ -1,11 +1,8 @@
-require_relative 'base_service'
-
 class DeleteTagService < BaseService
   def execute(tag_name)
     repository = project.repository
     tag = repository.find_tag(tag_name)
 
-    # No such tag
     unless tag
       return error('No such tag', 404)
     end
@@ -26,19 +23,20 @@ class DeleteTagService < BaseService
   end
 
   def error(message, return_code = 400)
-    out = super(message)
-    out[:return_code] = return_code
-    out
+    super(message).merge(return_code: return_code)
   end
 
   def success(message)
-    out = super()
-    out[:message] = message
-    out
+    super().merge(message: message)
   end
 
   def build_push_data(tag)
-    Gitlab::PushDataBuilder
-      .build(project, current_user, tag.target, Gitlab::Git::BLANK_SHA, "#{Gitlab::Git::TAG_REF_PREFIX}#{tag.name}", [])
+    Gitlab::DataBuilder::Push.build(
+      project,
+      current_user,
+      tag.dereferenced_target.sha,
+      Gitlab::Git::BLANK_SHA,
+      "#{Gitlab::Git::TAG_REF_PREFIX}#{tag.name}",
+      [])
   end
 end
