@@ -41,7 +41,7 @@ describe 'gitlab:app namespace rake task' do
 
     context 'gitlab version' do
       before do
-        allow(Dir).to receive(:glob).and_return([])
+        allow(Dir).to receive(:glob).and_return(['1_gitlab_backup.tar'])
         allow(Dir).to receive(:chdir)
         allow(File).to receive(:exist?).and_return(true)
         allow(Kernel).to receive(:system).and_return(true)
